require(['jquery', 'xwiki-events-bridge'], function($) {
  var enhanceSearchResultHighlights = function() {
    var highlights = $(this).removeClass('hidden').parent().prev('.search-result-highlights').addClass('preview');

    // Workaround for IE8 which doesn't support :first-of-type CSS selector.
    highlights.find('.search-result-highlight').first().addClass('first').parent('dd').addClass('first')
      .prev('dt').addClass('first');

    $(this).one('click', function(event) {
      event.preventDefault();
      $(event.target).remove();
      highlights.removeClass('preview');
    });
  };

  var collapseAllFacets = function(event) {
    event.preventDefault();
    $(event.target).closest('.search-facets').find('.search-facet').removeClass('expanded');
    $(event.target).hide().parent().children('.search-facets-action-expandAll').show();
  };

  var expandAllFacets = function(event) {
    event.preventDefault();
    $(event.target).closest('.search-facets').find('.search-facet').addClass('expanded');
    $(event.target).hide().parent().children('.search-facets-action-collapseAll').show();
  };

  var updateExpandCollapseAllFacetsState = function(facetsContainer) {
    var facetCount = facetsContainer.find('.search-facet').size();
    var expandedFacetCount = facetsContainer.find('.search-facet.expanded').size();
    if (expandedFacetCount > facetCount / 2) {
      facetsContainer.find('.search-facets-action-collapseAll').show()
        .parent().children('.search-facets-action-expandAll').hide();
    } else {
      facetsContainer.find('.search-facets-action-expandAll').show()
        .parent().children('.search-facets-action-collapseAll').hide();
    }
  };

  var addFacetValueCheckbox = function() {
    var checkBox = $(document.createElement('input')).attr('type', 'checkbox');
    checkBox.prop('checked', $(this).hasClass('selected'));
    // Add the 'checked' attribute so that it can be easily located with CSS.
    checkBox.prop('checked') && checkBox.attr('checked', 'checked');
    checkBox.click($.proxy(this, 'click'));
    // Remove the 'selected' class because the selected state is marked using the check box.
    $(this).removeClass('selected').before(checkBox);
  };

  var enhanceSearchFacets = function() {
    var facetsContainer = $(this);

    // Add the ability to collapse the facets pane on extra small screens (phones).
    facetsContainer.find('.search-facets-header').click(function() {
      facetsContainer.toggleClass('collapsed-xs');
    });

    // Global Expand/Collapse toggle.
    facetsContainer.find('.search-facets-action-collapseAll').removeClass('hidden').click(collapseAllFacets);
    facetsContainer.find('.search-facets-action-expandAll').removeClass('hidden').click(expandAllFacets);
    updateExpandCollapseAllFacetsState(facetsContainer);

    // Expand/Collapse toggle for each facet.
    facetsContainer.find('.search-facet-header').click(function(event) {
      $(event.target).parent('.search-facet').toggleClass('expanded');
      updateExpandCollapseAllFacetsState(facetsContainer);
    });

    // Add a check box before each facet value. We do this from JavaScript because the behaviour of a check box inside
    // a link is not consistent across different browsers (some follow the link when the check box is clicked, others
    // don't) and having the check box outside the facet value link requires JavaScript to synchronize them.
    facetsContainer.find('.search-facet-body a.itemName').each(addFacetValueCheckbox);
  };

  var getQueryString = function(url) {
    var queryIndex = url.indexOf('?');
    return queryIndex < 0 ? '' : url.substr(queryIndex + 1);
  };

  var getSearchUIState = function() {
    var expandedFacets = [];
    $('.search-facet.expanded').each(function() {
      expandedFacets.push($(this).attr('data-name'));
    });
    return {
      'expandedFacets': expandedFacets
    };
  };

  var setSearchUIState = function(state) {
    state = state || {};
    var expandedFacets = state.expandedFacets || [];
    $('.search-facet').not('.expanded').each(function() {
      if (expandedFacets.indexOf($(this).attr('data-name')) >= 0) {
        $(this).addClass('expanded');
      }
    });
  };

  var updateSearchUI = function(html) {
    $('.search-ui').replaceWith(html);
    enhanceSearchUI();
    $(document).trigger('xwiki:dom:updated', {'elements': $('.search-ui').toArray()});
  };

  var searchRequest = null;

  var pushSearchUIState = function(viewURL) {
    // If there is a request in progress, abort it to prevent its callback from being called.
    searchRequest && searchRequest.abort();
    $('.search-ui').attr('aria-busy', true);
    var getURL = XWiki.currentDocument.getURL('get', getQueryString(viewURL));
    searchRequest = $.get(getURL).done(function(html) {
      var state = getSearchUIState();
      updateSearchUI(html);
      setSearchUIState(state);
      window.history.replaceState && window.history.replaceState(state, document.title);
      // Make sure the browser address bar reflects the new state (and thus the new state can be bookmarked).
      window.history.pushState && window.history.pushState(state, document.title, viewURL);
    });
  };

  var reloadSearchUI = function(event) {
    event.preventDefault();
    var anchor = $(event.target).closest('a');
    anchor.size() > 0 && $(document).trigger('xwiki:search:update', anchor.attr('href'));
  };

  // Others (e.g. a custom facet) can trigger a search UI update by firing this event.
  $(document).on('xwiki:search:update', function(event, viewURL) {
    pushSearchUIState(viewURL);
  });

  $(window).on('popstate', function(event) {
    // If there is a request in progress, abort it to prevent its callback from being called.
    searchRequest && searchRequest.abort();
    $('.search-ui').attr('aria-busy', true);
    var state = event.originalEvent.state;
    var getURL = XWiki.currentDocument.getURL('get', window.location.search.substr(1));
    searchRequest = $.get(getURL).done(function(html) {
      updateSearchUI(html);
      setSearchUIState(state);
    });
  });

  // Move the skin extension imports brought by AJAX updates to the head of the page.
  $(document).on('xwiki:dom:updated', function(event, data) {
    // We want to move only the new imports. For this we need to collect the existing imports (once).
    var imports;
    // We don't add the imports directly to the document because jQuery will fetch the JavaScript files, including the
    // ones that have been already loaded. We append the imports to a detached element and add to the head of the page
    // only those that are new. See https://api.jquery.com/jQuery.getScript/#caching-requests .
    $(document.createElement('div')).append($(data.elements).find('noscript.skin-extension-imports').text())
      .find('link, script').each(function() {
        // Collect the existing imports once.
        imports = imports || $('head').find('link, script').map(function () {
          return $(this).attr('href') || $(this).attr('src');
        }).get();
        var url = $(this).attr('href') || $(this).attr('src');
        // Move to the head of the page only the new imports.
        if (url && imports.indexOf(url) < 0) {
          $('head').append(this);
        }
      });
    // Make sure this code is not executed twice.
    $(data.elements).find('noscript.skin-extension-imports').remove();
  });

  var enhanceSearchUI = function() {
    $('.search-result-highlightAll').each(enhanceSearchResultHighlights);
    $('.search-facets').each(enhanceSearchFacets);
    $([
      '.search-results-sort a.sort-item',
      '.search-options a.options-item',
      '.pagination a',
      '.controlPagination a',
      'a.search-facets-action-reset',
      '.search-facet a.itemName',
      '.search-facet-body a.more',
      '.search-facet-body .selectedLocation a'
    ].join(',')).click(reloadSearchUI);
  };

  enhanceSearchUI();
});
